/*
Run this script only if you have one or more custom MS SQL log channels defined (in addition
to the Default SQL Channel). While it is not very common to have additional ones defined,
perform the following task to verify if you do:
		1. Using a compatible web browser, open the Web Administration Console.
		2. Go to the Log Tree and click the Channels plus sign (+) to expand Channels.
		3. Do one of the following:
			- If you find one or more custom MS SQL channels (in addition to the Default SQL
			  Channel), run this script.
			- If you don't find a custom SQL channel, then you do not need to run this script.
																									
NOTE: Trust Protection Platform will run the appropriate script on the Default MS SQL Channel.
*/

/***** Modify this variable for the table to be migrated *****/
DECLARE @tablename VARCHAR(108) = 'Replace';

/***** Do not modify below this line *****/
--------------------------------------------------
DECLARE @channelname VARCHAR(108);
DECLARE @Message VARCHAR(1000);
DECLARE @SQLString NVARCHAR(4000);
DECLARE @SQLString2 NVARCHAR(4000);

BEGIN
	BEGIN TRANSACTION;

	BEGIN TRY
		SET @Message =  'DATE : ' + CONVERT(NVARCHAR(30),SYSDATETIME(),20) + N' - LOG PARAMS computed column fix.';
		RAISERROR(@Message, 0, 1) WITH NOWAIT;
		
		SET @SQLString = N'
		IF EXISTS(SELECT * FROM sys.objects WHERE parent_object_id = OBJECT_ID(''[dbo].[DAL_T_'+@tablename+'_LOG_PARAMS]'') and name = ''UQ_DAL_T_'+@tablename+'_LOG_PARAMS'') 
		BEGIN 
			ALTER TABLE [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] DROP CONSTRAINT [UQ_DAL_T_'+@tablename+'_LOG_PARAMS] 
		END';

		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		IF COL_LENGTH(''[dbo].[DAL_T_'+@tablename+'_LOG_PARAMS]'', ''TEXT_COMPUTED'') IS NOT NULL BEGIN
			ALTER TABLE [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] DROP COLUMN [TEXT_COMPUTED];
		END';

		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		ALTER TABLE [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] ADD [TEXT_COMPUTED] AS (CONVERT([nvarchar](64),HASHBYTES(''SHA1'',isnull([TEXT_1],''NULL'')+isnull([TEXT_2],''NULL'')),2));';
						
		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		ALTER TABLE [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] ADD  CONSTRAINT [UQ_DAL_T_'+@tablename+'_LOG_PARAMS] UNIQUE NONCLUSTERED ([ADDRESS_1] ASC, [ADDRESS_2] ASC, [VALUE_1] ASC,[VALUE_2] ASC,[TEXT_COMPUTED] );';
		
		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		IF EXISTS(SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(''[dbo].[DAL_T_'+@tablename+'_LOG_PARAMS]'') and name = ''DAL_X_'+@tablename+'_LOG_PRMS'') 
		BEGIN 
			DROP INDEX [DAL_X_'+@tablename+'_LOG_PRMS] ON [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS];
		END';

		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		IF NOT EXISTS(SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(''[dbo].[DAL_T_'+@tablename+'_LOG_PARAMS]'') and name = ''DAL_X_'+@tablename+'_LOG_TXT1'') 
		BEGIN						
			CREATE INDEX [DAL_X_'+@tablename+'_LOG_TXT1] ON [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] ([TEXT_1]);					
		END';

		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		IF (OBJECT_ID (''[dbo].[DAL_SP_V_'+@tablename+'_LOG_BI]'', ''P'') IS NULL)
		BEGIN
			EXECUTE dbo.sp_executesql @statement = N''CREATE PROCEDURE [dbo].[DAL_SP_V_'+@tablename+'_LOG_BI] AS SET NOCOUNT ON;'';
		END';

		execute dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N'
		ALTER PROCEDURE [dbo].[DAL_SP_V_'+@tablename+'_LOG_BI] (
			@InRows [dbo].[DAL_V_LOG_TYPE] READONLY
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;
	
			SET XACT_ABORT, NOCOUNT ON;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @InsertedData TABLE(PK INT IDENTITY(0,1) NOT NULL, DATA_ID int NOT NULL);
				DECLARE @InsertedIds TABLE(PK INT IDENTITY NOT NULL, LOG_ID int NOT NULL);

				IF @v_has_outer_transaction = 0
				BEGIN
					RAISERROR (N''No active transaction!'', 10, 1);
					RETURN -1;
				END

				-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_SOURCE_IP
				INSERT INTO [dbo].[DAL_T_'+@tablename+'_LOG_SOURCE_IP] ([SOURCE_IP])
					SELECT DISTINCT ir.[SOURCE_IP]
						FROM @InRows ir WHERE NOT EXISTS (
							SELECT TOP 1 1 FROM DAL_T_'+@tablename+'_LOG_SOURCE_IP t0 WHERE (t0.SOURCE_IP = ir.SOURCE_IP OR (ir.SOURCE_IP IS NULL AND t0.SOURCE_IP IS NULL))
						);

				-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_COMP
				INSERT INTO [dbo].[DAL_T_'+@tablename+'_LOG_COMP] (COMPONENT, COMPONENT_ID, COMPONENT_SUBSYSTEM)
					SELECT DISTINCT ir.COMPONENT, ir.COMPONENT_ID, ir.COMPONENT_SUBSYSTEM
						FROM @InRows ir WHERE NOT EXISTS (
							SELECT TOP 1 1 FROM DAL_T_'+@tablename+'_LOG_COMP t1 WHERE (t1.COMPONENT = ir.COMPONENT OR (ir.COMPONENT IS NULL AND t1.COMPONENT IS NULL))
								AND (t1.COMPONENT_ID = ir.COMPONENT_ID OR (ir.COMPONENT_ID IS NULL AND t1.COMPONENT_ID IS NULL))
								AND (t1.COMPONENT_SUBSYSTEM = ir.COMPONENT_SUBSYSTEM OR (ir.COMPONENT_SUBSYSTEM IS NULL AND t1.COMPONENT_SUBSYSTEM IS NULL))
						);

				-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_PARAMS
				INSERT INTO [dbo].[DAL_T_'+@tablename+'_LOG_PARAMS] (ADDRESS_1, ADDRESS_2, TEXT_1, TEXT_2, VALUE_1, VALUE_2)
					SELECT DISTINCT ir.ADDRESS_1, ir.ADDRESS_2, ir.TEXT_1, ir.TEXT_2, ir.VALUE_1, ir.VALUE_2
						FROM @InRows ir WHERE NOT EXISTS (
							SELECT TOP 1 1 FROM DAL_T_'+@tablename+'_LOG_PARAMS t2 WHERE (t2.ADDRESS_1 = ir.ADDRESS_1 OR (ir.ADDRESS_1 IS NULL AND t2.ADDRESS_1 IS NULL))
								AND (t2.ADDRESS_2 = ir.ADDRESS_2 OR (ir.ADDRESS_2 IS NULL AND t2.ADDRESS_2 IS NULL))
								AND (t2.VALUE_1 = ir.VALUE_1 OR (ir.VALUE_1 IS NULL AND t2.VALUE_1 IS NULL))
								AND (t2.VALUE_2 = ir.VALUE_2 OR (ir.VALUE_2 IS NULL AND t2.VALUE_2 IS NULL))
								AND (t2.TEXT_COMPUTED = CONVERT([nvarchar](64),HASHBYTES(''SHA1'',isnull(ir.TEXT_1,''NULL'')+isnull(ir.TEXT_2,''NULL'')),2))
						);
		';

		SET @SQLString2 = N'
				-- insert all denormalized data DAL_T_<CHANNEL>_LOG_DATA
				INSERT INTO [dbo].[DAL_T_'+@tablename+'_LOG_DATA] (DATA_ARRAY, DATA_SIZE, MIME_TYPE)
					OUTPUT INSERTED.DATA_ID INTO @InsertedData
					SELECT  ir.DATA_ARRAY, ir.DATA_SIZE, ir.MIME_TYPE
						FROM @InRows ir
						ORDER BY INDEX_ORDER ASC;

				INSERT INTO [dbo].[DAL_T_'+@tablename+'_LOG_MESSAGE] (LOG_MESSAGE_ID, CLIENT_TIMESTAMP, COMP_ID, DATA_ID, EVENT_ID, GROUP_ID, PARAMS_ID, SERVER_TIMESTAMP, SEVERITY, SOURCE_IP_ID)
					OUTPUT INSERTED.LOG_MESSAGE_ID INTO @InsertedIds
					SELECT (SELECT t4.DATA_ID
							FROM @InsertedData t4
							WHERE (t4.PK = ir.INDEX_ORDER)),
						ir.CLIENT_TIMESTAMP,
						t1.COMP_ID,
						t3.DATA_ID,
						ir.EVENT_ID,
						ir.GROUP_ID,
						t2.PARAMS_ID,
						ir.SERVER_TIMESTAMP,
						ir.SEVERITY,
						t0.SOURCE_IP_ID
					FROM @InRows ir
					INNER JOIN DAL_T_'+@tablename+'_LOG_SOURCE_IP t0
						ON t0.SOURCE_IP = ir.SOURCE_IP OR (t0.SOURCE_IP IS NULL AND ir.SOURCE_IP IS NULL)
					INNER JOIN DAL_T_'+@tablename+'_LOG_COMP t1
						ON (t1.COMPONENT = ir.COMPONENT OR (t1.COMPONENT IS NULL AND ir.COMPONENT IS NULL))
							AND (t1.COMPONENT_ID = ir.COMPONENT_ID OR (t1.COMPONENT_ID IS NULL AND ir.COMPONENT_ID IS NULL))
							AND (t1.COMPONENT_SUBSYSTEM = ir.COMPONENT_SUBSYSTEM OR (t1.COMPONENT_SUBSYSTEM IS NULL AND ir.COMPONENT_SUBSYSTEM IS NULL))
					INNER JOIN DAL_T_'+@tablename+'_LOG_PARAMS t2
						ON (t2.ADDRESS_1 = ir.ADDRESS_1 OR (t2.ADDRESS_1 IS NULL AND ir.ADDRESS_1 IS NULL))
							AND (t2.ADDRESS_2 = ir.ADDRESS_2 OR (t2.ADDRESS_2 IS NULL AND ir.ADDRESS_2 IS NULL))
							AND (t2.VALUE_1 = ir.VALUE_1 OR (t2.VALUE_1 IS NULL AND ir.VALUE_1 IS NULL))
							AND (t2.VALUE_2 = ir.VALUE_2 OR (t2.VALUE_2 IS NULL AND ir.VALUE_2 IS NULL))
							AND (t2.TEXT_COMPUTED = CONVERT([nvarchar](64),HASHBYTES(''SHA1'',isnull(ir.TEXT_1,''NULL'')+isnull(ir.TEXT_2,''NULL'')),2))
					INNER JOIN @InsertedData t3
						ON t3.PK = ir.INDEX_ORDER
					ORDER BY INDEX_ORDER ASC;

				SELECT LOG_ID FROM @InsertedIds ORDER BY PK ASC;																											
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);  
				DECLARE @v_error_severity INT;  
				DECLARE @v_error_state INT;  

				SELECT   
					@v_error_message = ERROR_MESSAGE(),  
					@v_error_severity = ERROR_SEVERITY(),  
					@v_error_state = ERROR_STATE();  

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
			END CATCH
		END;
		';

		EXEC (@SQLString + @SQLString2);

		execute dbo.sp_executesql @statement = N'
		IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_CREATE_LOG_CHANNEL]'', ''P'') IS NULL)
		BEGIN
			EXECUTE dbo.sp_executesql @statement = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_CREATE_LOG_CHANNEL] AS SET NOCOUNT ON;'';
		END';

		EXECUTE dbo.sp_executesql @statement = N'ALTER PROCEDURE [dbo].[DAL_LOG_SP_CREATE_LOG_CHANNEL] (
		@ChannelName nvarchar(32)) AS
		BEGIN
			DECLARE @SQLString NVARCHAR(4000);
			DECLARE @SQLString2 NVARCHAR(4000);
			DECLARE @SQLString3 NVARCHAR(4000);
			DECLARE @SQLString4 NVARCHAR(4000);
			SET @SQLString = N''CREATE TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
				(
					[LOG_MESSAGE_ID] bigint NOT NULL ,
					[CLIENT_TIMESTAMP] datetime NULL ,
					[COMP_ID] bigint NOT NULL ,
					[DATA_ID] bigint NOT NULL ,
					[EVENT_ID] int NULL ,
					[GROUP_ID] int NULL ,
					[PARAMS_ID] bigint NOT NULL ,
					[SERVER_TIMESTAMP] datetime NULL ,
					[SEVERITY] smallint NULL ,
					[SOURCE_IP_ID] int NOT NULL ,
					CONSTRAINT [DAL_PK_''+@ChannelName+''_LOG_MESSAGE] PRIMARY KEY NONCLUSTERED ( [LOG_MESSAGE_ID] )
				);''
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP]
			(
				[COMP_ID] bigint identity NOT NULL ,
				[COMPONENT] nvarchar(255) NULL ,
				[COMPONENT_ID] bigint NULL ,
				[COMPONENT_SUBSYSTEM] int NULL ,
				CONSTRAINT [DAL_PK_''+@ChannelName+''_LOG_COMP] PRIMARY KEY NONCLUSTERED ( [COMP_ID] )
			);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;
						
		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP]
			ADD CONSTRAINT [UQ_DAL_T_''+@ChannelName+''_LOG_COMP]
			UNIQUE (COMPONENT, COMPONENT_ID, COMPONENT_SUBSYSTEM);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA]
			(
				[DATA_ID] bigint identity NOT NULL ,
				[DATA_ARRAY] varbinary(max) NULL ,
				[DATA_SIZE] int NULL ,
				[MIME_TYPE] smallint NULL ,
				CONSTRAINT [DAL_PK_''+@ChannelName+''_LOG_DATA] PRIMARY KEY NONCLUSTERED ( [DATA_ID] )
			);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS]
			(
				[PARAMS_ID] bigint identity NOT NULL ,
				[ADDRESS_1] varbinary(16) NULL ,
				[ADDRESS_2] varbinary(16) NULL ,
				[TEXT_1] nvarchar(255) NULL ,
				[TEXT_2] nvarchar(255) NULL ,
				[VALUE_1] bigint NULL ,
				[VALUE_2] bigint NULL ,
				[TEXT_COMPUTED]  AS (CONVERT([nvarchar](64),HASHBYTES(''''SHA1'''',isnull([TEXT_1],''''NULL'''')+isnull([TEXT_2],''''NULL'''')),2)),
				CONSTRAINT [DAL_PK_''+@ChannelName+''_LOG_PARAMS] PRIMARY KEY NONCLUSTERED ( [PARAMS_ID] ),
				CONSTRAINT [UQ_DAL_T_''+@ChannelName+''_LOG_PARAMS] UNIQUE NONCLUSTERED ([ADDRESS_1] ASC, [ADDRESS_2] ASC, [VALUE_1] ASC,[VALUE_2] ASC,[TEXT_COMPUTED] )
			);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;	

		SET @SQLString = N''CREATE TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP]
			(
				[SOURCE_IP_ID] int identity NOT NULL ,
				[SOURCE_IP] varbinary(16) NULL ,							
				CONSTRAINT [DAL_PK_''+@ChannelName+''_LOG_SOURCE_IP] PRIMARY KEY NONCLUSTERED ( [SOURCE_IP_ID] )
			);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;
						
		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP]
			ADD CONSTRAINT [UQ_DAL_T_''+@ChannelName+''_LOG_SOURCE_IP]
			UNIQUE (SOURCE_IP);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
			WITH CHECK ADD 	CONSTRAINT [DAL_FK_''+@ChannelName+''_LOG_MESSAGE_COMP_ID]
			FOREIGN KEY([COMP_ID])
			REFERENCES [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] ([COMP_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
			WITH CHECK ADD CONSTRAINT [DAL_FK_''+@ChannelName+''_LOG_MESSAGE_DATA_ID]
			FOREIGN KEY([DATA_ID])
			REFERENCES [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] ([DATA_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
			WITH CHECK ADD CONSTRAINT [DAL_FK_''+@ChannelName+''_LOG_MESSAGE_PARAMS_ID]
			FOREIGN KEY([PARAMS_ID])
			REFERENCES [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([PARAMS_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''ALTER TABLE [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
			WITH CHECK ADD CONSTRAINT [DAL_FK_''+@ChannelName+''_LOG_MESSAGE_SOURCE_IP_ID]
			FOREIGN KEY([SOURCE_IP_ID])
			REFERENCES [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] ([SOURCE_IP_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE VIEW [dbo].[DAL_V_''+@ChannelName+''_LOG_MESSAGE] WITH SCHEMABINDING AS
			SELECT t2.ADDRESS_1, t2.ADDRESS_2, bt.CLIENT_TIMESTAMP, bt.COMP_ID, t1.COMPONENT, t1.COMPONENT_ID, t1.COMPONENT_SUBSYSTEM, t3.DATA_ARRAY, bt.DATA_ID, t3.DATA_SIZE, bt.EVENT_ID, bt.GROUP_ID, bt.LOG_MESSAGE_ID, t3.MIME_TYPE, bt.PARAMS_ID, bt.SERVER_TIMESTAMP, bt.SEVERITY, t0.SOURCE_IP, t2.TEXT_1, t2.TEXT_2, t2.VALUE_1, t2.VALUE_2
			FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] bt
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] t0 ON bt.SOURCE_IP_ID=t0.SOURCE_IP_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] t1 ON bt.COMP_ID=t1.COMP_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] t2 ON bt.PARAMS_ID=t2.PARAMS_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] t3 ON bt.DATA_ID=t3.DATA_ID;'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE VIEW [dbo].[''+@ChannelName+''] WITH SCHEMABINDING AS
			SELECT t0.SOURCE_IP AS [SourceIP], bt.CLIENT_TIMESTAMP AS [ClientTimestamp], bt.SERVER_TIMESTAMP AS [ServerTimestamp], t1.COMPONENT AS [Component], t1.COMPONENT_ID AS [ComponentId],
				t1.COMPONENT_SUBSYSTEM AS [ComponentSubsystem], bt.EVENT_ID AS [EventID], bt.SEVERITY AS [Severity], bt.GROUP_ID AS [GroupingID], t2.TEXT_1 AS [Text1], t2.TEXT_2 AS [Text2],
				t2.VALUE_1 AS [Value1], t2.VALUE_2 AS [Value2], t2.ADDRESS_1 AS [Address1], t2.ADDRESS_2 AS [Address2], t3.MIME_TYPE AS [MIMEType], t3.DATA_SIZE AS [DataSize], t3.DATA_ARRAY AS [Data]
			FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] bt
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] t0 ON bt.SOURCE_IP_ID=t0.SOURCE_IP_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] t1 ON bt.COMP_ID=t1.COMP_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] t2 ON bt.PARAMS_ID=t2.PARAMS_ID
				INNER JOIN [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] t3 ON bt.DATA_ID=t3.DATA_ID;'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		-- LOG_MESSAGE TABLE INDEXES:
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_SVRTY] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([SEVERITY],[CLIENT_TIMESTAMP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_CLTTS] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([CLIENT_TIMESTAMP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_EVTID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([EVENT_ID],[CLIENT_TIMESTAMP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_GRPID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([GROUP_ID],[CLIENT_TIMESTAMP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_SRVTS] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([SERVER_TIMESTAMP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		-- LOG_MESSAGE FOREIGN KEY INDEXES:
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_CMPID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([COMP_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_DATID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([DATA_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_PRMID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([PARAMS_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_SRCID] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] ([SOURCE_IP_ID]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		-- LOG_COMP TABLE INDEXES:
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_CMP] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] ([COMPONENT]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_CMPS] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] ([COMPONENT_ID],[COMPONENT_SUBSYSTEM],[COMPONENT]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		-- LOG_DATA TABLE INDEXES:
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_DATSZ] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] ([DATA_SIZE],[MIME_TYPE]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		-- LOG_SOURCE_IP TABLE INDEXES:
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_SRCIP] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] ([SOURCE_IP]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_ADR1] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([ADDRESS_1]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;
						
		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_ADR2] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([ADDRESS_2]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_TXT1] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([TEXT_1]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_TXT2] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([TEXT_2]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_VAL1] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([VALUE_1]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		SET @SQLString = N''CREATE INDEX [DAL_X_''+@ChannelName+''_LOG_VAL2] ON [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] ([VALUE_2]);'';
		EXECUTE dbo.sp_executesql @statement = @SQLString;

		IF (OBJECT_ID (''[dbo].[DAL_SP_V_''+@ChannelName+''_LOG_BI]'', ''P'') IS NULL)
		BEGIN
			SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_SP_V_''+@ChannelName+''_LOG_BI] AS SET NOCOUNT ON;''
			EXECUTE dbo.sp_executesql @statement = @SQLString;
		END

		SET @SQLString = N''
			ALTER PROCEDURE [dbo].[DAL_SP_V_''+@ChannelName+''_LOG_BI] (
				@InRows [dbo].[DAL_V_LOG_TYPE] READONLY
			)
			AS
			BEGIN
				DECLARE @v_has_outer_transaction BIT;
		
				SET XACT_ABORT, NOCOUNT ON;

				SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

				BEGIN TRY
					DECLARE @InsertedData TABLE(PK INT IDENTITY(0,1) NOT NULL, DATA_ID int NOT NULL);
					DECLARE @InsertedIds TABLE(PK INT IDENTITY NOT NULL, LOG_ID int NOT NULL);

					IF @v_has_outer_transaction = 0
					BEGIN
						RAISERROR (N''''No active transaction!'''', 10, 1);
						RETURN -1;
					END

					-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_SOURCE_IP
					INSERT INTO [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] ([SOURCE_IP])
						SELECT DISTINCT ir.[SOURCE_IP]
							FROM @InRows ir WHERE NOT EXISTS (
								SELECT TOP 1 1 FROM DAL_T_''+@ChannelName+''_LOG_SOURCE_IP t0 WHERE (t0.SOURCE_IP = ir.SOURCE_IP OR (ir.SOURCE_IP IS NULL AND t0.SOURCE_IP IS NULL))
							);

					-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_COMP
					INSERT INTO [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] (COMPONENT, COMPONENT_ID, COMPONENT_SUBSYSTEM)
						SELECT DISTINCT ir.COMPONENT, ir.COMPONENT_ID, ir.COMPONENT_SUBSYSTEM
							FROM @InRows ir WHERE NOT EXISTS (
								SELECT TOP 1 1 FROM DAL_T_''+@ChannelName+''_LOG_COMP t1 WHERE (t1.COMPONENT = ir.COMPONENT OR (ir.COMPONENT IS NULL AND t1.COMPONENT IS NULL))
									AND (t1.COMPONENT_ID = ir.COMPONENT_ID OR (ir.COMPONENT_ID IS NULL AND t1.COMPONENT_ID IS NULL))
									AND (t1.COMPONENT_SUBSYSTEM = ir.COMPONENT_SUBSYSTEM OR (ir.COMPONENT_SUBSYSTEM IS NULL AND t1.COMPONENT_SUBSYSTEM IS NULL))
							);

					-- insert all distinct normalized data DAL_T_<CHANNEL>_LOG_PARAMS
					INSERT INTO [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] (ADDRESS_1, ADDRESS_2, TEXT_1, TEXT_2, VALUE_1, VALUE_2)
						SELECT DISTINCT ir.ADDRESS_1, ir.ADDRESS_2, ir.TEXT_1, ir.TEXT_2, ir.VALUE_1, ir.VALUE_2
							FROM @InRows ir WHERE NOT EXISTS (
								SELECT TOP 1 1 FROM DAL_T_''+@ChannelName+''_LOG_PARAMS t2 WHERE (t2.ADDRESS_1 = ir.ADDRESS_1 OR (ir.ADDRESS_1 IS NULL AND t2.ADDRESS_1 IS NULL))
									AND (t2.ADDRESS_2 = ir.ADDRESS_2 OR (ir.ADDRESS_2 IS NULL AND t2.ADDRESS_2 IS NULL))
									AND (t2.VALUE_1 = ir.VALUE_1 OR (ir.VALUE_1 IS NULL AND t2.VALUE_1 IS NULL))
									AND (t2.VALUE_2 = ir.VALUE_2 OR (ir.VALUE_2 IS NULL AND t2.VALUE_2 IS NULL))
									AND (t2.TEXT_COMPUTED = CONVERT([nvarchar](64),HASHBYTES(''''SHA1'''',isnull(ir.TEXT_1,''''NULL'''')+isnull(ir.TEXT_2,''''NULL'''')),2))
							);
			'';

			SET @SQLString2 = N''
				-- insert all denormalized data DAL_T_<CHANNEL>_LOG_DATA
				INSERT INTO [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] (DATA_ARRAY, DATA_SIZE, MIME_TYPE)
					OUTPUT INSERTED.DATA_ID INTO @InsertedData
					SELECT  ir.DATA_ARRAY, ir.DATA_SIZE, ir.MIME_TYPE
						FROM @InRows ir
						ORDER BY INDEX_ORDER ASC;
										
				INSERT INTO [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] (LOG_MESSAGE_ID, CLIENT_TIMESTAMP, COMP_ID, DATA_ID, EVENT_ID, GROUP_ID, PARAMS_ID, SERVER_TIMESTAMP, SEVERITY, SOURCE_IP_ID)
					OUTPUT INSERTED.LOG_MESSAGE_ID INTO @InsertedIds
					SELECT (SELECT t4.DATA_ID
							FROM @InsertedData t4
							WHERE (t4.PK = ir.INDEX_ORDER)),
						ir.CLIENT_TIMESTAMP,
						t1.COMP_ID,
						t3.DATA_ID,
						ir.EVENT_ID,
						ir.GROUP_ID,
						t2.PARAMS_ID,
						ir.SERVER_TIMESTAMP,
						ir.SEVERITY,
						t0.SOURCE_IP_ID
					FROM @InRows ir
					INNER JOIN DAL_T_''+@ChannelName+''_LOG_SOURCE_IP t0
						ON t0.SOURCE_IP = ir.SOURCE_IP OR (t0.SOURCE_IP IS NULL AND ir.SOURCE_IP IS NULL)
					INNER JOIN DAL_T_''+@ChannelName+''_LOG_COMP t1
						ON (t1.COMPONENT = ir.COMPONENT OR (t1.COMPONENT IS NULL AND ir.COMPONENT IS NULL))
							AND (t1.COMPONENT_ID = ir.COMPONENT_ID OR (t1.COMPONENT_ID IS NULL AND ir.COMPONENT_ID IS NULL))
							AND (t1.COMPONENT_SUBSYSTEM = ir.COMPONENT_SUBSYSTEM OR (t1.COMPONENT_SUBSYSTEM IS NULL AND ir.COMPONENT_SUBSYSTEM IS NULL))
					INNER JOIN DAL_T_''+@ChannelName+''_LOG_PARAMS t2
						ON (t2.ADDRESS_1 = ir.ADDRESS_1 OR (t2.ADDRESS_1 IS NULL AND ir.ADDRESS_1 IS NULL))
							AND (t2.ADDRESS_2 = ir.ADDRESS_2 OR (t2.ADDRESS_2 IS NULL AND ir.ADDRESS_2 IS NULL))
							AND (t2.VALUE_1 = ir.VALUE_1 OR (t2.VALUE_1 IS NULL AND ir.VALUE_1 IS NULL))
							AND (t2.VALUE_2 = ir.VALUE_2 OR (t2.VALUE_2 IS NULL AND ir.VALUE_2 IS NULL))
							AND (t2.TEXT_COMPUTED = CONVERT([nvarchar](64),HASHBYTES(''''SHA1'''',isnull(ir.TEXT_1,''''NULL'''')+isnull(ir.TEXT_2,''''NULL'''')),2))
					INNER JOIN @InsertedData t3
						ON t3.PK = ir.INDEX_ORDER
					ORDER BY INDEX_ORDER ASC;

				SELECT LOG_ID FROM @InsertedIds ORDER BY PK ASC;																											
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);  
				DECLARE @v_error_severity INT;  
				DECLARE @v_error_state INT;  

				SELECT   
					@v_error_message = ERROR_MESSAGE(),  
					@v_error_severity = ERROR_SEVERITY(),  
					@v_error_state = ERROR_STATE();  

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
			END CATCH
		END;
	'';

	EXEC (@SQLString + @SQLString2);

	IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRY]'', ''P'') IS NULL)
	BEGIN
		SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRY] AS SET NOCOUNT ON;''
		EXECUTE dbo.sp_executesql @statement = @SQLString; 
	END

	SET @SQLString = N''
		-- The purpose of this procedure is to delete log entries that have expired.
		-- This procedure should only be called with a world wide exclusitiviy lock managed by the caller.
		-- Expired means older than a given date.  The procedure removes no more than 500 message table rows per call.
		ALTER PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRY] (
			@p_threshold DATETIME,					-- Expiration threshold.  Entries older than this will be deleted.
			@o_deleted_message_rows INT OUTPUT,		-- Returns the number of message rows deleted.
			@o_deleted_data_rows INT OUTPUT,		-- Returns the number of data rows deleted.
			@o_deleted_params_rows INT OUTPUT,		-- Returns the number of params rows deleted.
			@o_deleted_comp_rows INT OUTPUT,		-- Returns the number of comp rows deleted.
			@o_deleted_source_ip_rows INT OUTPUT	-- Returns the number of source ip rows deleted.
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;

			SET XACT_ABORT, NOCOUNT ON;

			SET @o_deleted_message_rows = -1;
			SET @o_deleted_data_rows = -1;
			SET @o_deleted_params_rows = -1;
			SET @o_deleted_comp_rows = -1;
			SET @o_deleted_source_ip_rows = -1;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @v_foreign_keys TABLE (
					data_id BIGINT UNIQUE NOT NULL,
					params_id BIGINT NOT NULL, 
					comp_id BIGINT NOT NULL, 
					source_ip_id INT NOT NULL
				);
				DECLARE @v_deleted_rows INT;

				IF @v_has_outer_transaction = 0
				BEGIN
					BEGIN TRANSACTION;
				END;

				DELETE TOP(500)
					FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE]
					OUTPUT deleted.[DATA_ID], deleted.[PARAMS_ID], deleted.[COMP_ID], deleted.[SOURCE_IP_ID] INTO @v_foreign_keys ( data_id, params_id, comp_id, source_ip_id )
					WHERE [CLIENT_TIMESTAMP] < @p_threshold;

				SET @v_deleted_rows = @@ROWCOUNT;

				IF @v_has_outer_transaction = 0
				BEGIN
					COMMIT TRANSACTION;
				END;

				SET @o_deleted_message_rows = @v_deleted_rows;

				-- If no message rows were deleted then there is no need to cleanup any newly created orphan rows.
				IF @o_deleted_message_rows > 0
				BEGIN
					DECLARE @v_comp_keys TABLE (
						comp_id BIGINT UNIQUE NOT NULL
					);
					DECLARE @v_params_keys TABLE (
						params_id BIGINT UNIQUE NOT NULL
					);
					DECLARE @v_source_ip_keys TABLE (
						source_ip_id BIGINT UNIQUE NOT NULL
					);

					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					-- Remove any data rows that are no longer referenced (e.g. orphans).
					DELETE DAL_T_''+@ChannelName+''_LOG_DATA 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] AS d 
							INNER JOIN @v_foreign_keys AS fk ON [d].DATA_ID = [fk].[data_id];

					SET @v_deleted_rows = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_data_rows = @v_deleted_rows;
		'';

	SET @SQLString2 = N''

					-- Remove any params rows that are no longer referenced (e.g. orphans).
					INSERT INTO @v_params_keys ( params_id ) 
						SELECT DISTINCT [params_id] FROM @v_foreign_keys;

					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_PARAMS 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] AS p 
							INNER JOIN @v_params_keys AS pk ON [p].[PARAMS_ID] = [pk].[params_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [p].[PARAMS_ID] = [msg].[PARAMS_ID]
						);

					SET @v_deleted_rows = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_params_rows = @v_deleted_rows;

					-- Remove any comp rows that are no longer referenced (e.g. orphans).
					INSERT INTO @v_comp_keys ( comp_id ) 
						SELECT DISTINCT [comp_id] FROM @v_foreign_keys;

					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_COMP 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] AS c 
							INNER JOIN @v_comp_keys AS ck ON [c].[COMP_ID] = [ck].[comp_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [c].[COMP_ID] = [msg].[COMP_ID] 
						);

					SET @v_deleted_rows = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_comp_rows = @v_deleted_rows;

					-- Remove any source ip rows that are no longer referenced (e.g. orphans).
					INSERT INTO @v_source_ip_keys ( source_ip_id ) 
						SELECT DISTINCT [source_ip_id] FROM @v_foreign_keys;

					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_SOURCE_IP 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] AS s 
							INNER JOIN @v_source_ip_keys AS sk ON [s].[SOURCE_IP_ID] = [sk].[source_ip_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [s].[SOURCE_IP_ID] = [msg].[SOURCE_IP_ID] 
						);

					SET @v_deleted_rows = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_source_ip_rows = @v_deleted_rows;
				END
				ELSE
				BEGIN
					SET @o_deleted_data_rows = 0;
					SET @o_deleted_params_rows = 0;
					SET @o_deleted_comp_rows = 0;
					SET @o_deleted_source_ip_rows = 0;
				END;

				RETURN 0;
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);
				DECLARE @v_error_severity INT;
				DECLARE @v_error_state INT;
				DECLARE @v_error INT;

				SELECT @v_error = @@ERROR, @v_error_message = ERROR_MESSAGE(), @v_error_severity = ERROR_SEVERITY(), @v_error_state = ERROR_STATE();

				IF @v_has_outer_transaction = 0
				BEGIN
					IF XACT_STATE() <> 0 
					BEGIN
						ROLLBACK TRANSACTION;
					END;
				END;

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
				RETURN -@v_error;
			END CATCH;
		END;
	'';

	EXEC (@SQLString + @SQLString2);

	IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_DATA]'', ''P'') IS NULL)
	BEGIN
		SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_DATA] AS SET NOCOUNT ON;''
		EXECUTE dbo.sp_executesql @statement = @SQLString; 
	END

	SET @SQLString = N''
		-- The purpose of this procedure is to delete orphaned log data entries.
		-- This procedure should only be called with a world wide exclusitiviy lock managed by the caller.
		ALTER PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_DATA] (
			@o_deleted_data_rows INT OUTPUT		-- Returns the number of data rows deleted.
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;
	
			SET XACT_ABORT, NOCOUNT ON;

			SET @o_deleted_data_rows = -1;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @v_data_keys TABLE (
					data_id BIGINT UNIQUE NOT NULL
				);
				DECLARE @v_orphans INT;

				INSERT INTO @v_data_keys ( data_id ) 
					SELECT TOP(500) [d].[DATA_ID] AS data_id 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] AS d WITH (NOLOCK) 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [d].[DATA_ID] = [msg].[DATA_ID] 
						);

				SET @v_orphans = @@ROWCOUNT;
	
				IF @v_orphans > 0
				BEGIN
					-- This time (with locks), try to delete our orphan candidates found during the non-blocking scan while transactionally ensuring they are truly orphans.
					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_DATA 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_DATA] AS d 
							INNER JOIN @v_data_keys AS dk ON [d].[DATA_ID] = [dk].[data_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [d].[DATA_ID] = [msg].[DATA_ID] 
						);

					SET @v_orphans = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_data_rows = @v_orphans;
				END
				ELSE
				BEGIN
					SET @o_deleted_data_rows = 0;
				END;

				RETURN 0;
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);
				DECLARE @v_error_severity INT;
				DECLARE @v_error_state INT;
				DECLARE @v_error INT;

				SELECT @v_error = @@ERROR, @v_error_message = ERROR_MESSAGE(), @v_error_severity = ERROR_SEVERITY(), @v_error_state = ERROR_STATE();

				IF @v_has_outer_transaction = 0
				BEGIN
					IF XACT_STATE() <> 0 
					BEGIN
						ROLLBACK TRANSACTION;
					END;
				END;

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
				RETURN -@v_error;
			END CATCH;
		END;
		'';

	EXECUTE dbo.sp_executesql @statement = @SQLString;

	IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_PARAMS]'', ''P'') IS NULL)
	BEGIN
		SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_PARAMS] AS SET NOCOUNT ON;''
		EXECUTE dbo.sp_executesql @statement = @SQLString; 
	END

	SET @SQLString = N''
		-- The purpose of this procedure is to delete orphaned log params entries.
		-- This procedure should only be called with a world wide exclusitiviy lock managed by the caller.
		ALTER PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_PARAMS] (
			@o_deleted_params_rows INT OUTPUT		-- Returns the number of params rows deleted.
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;
	
			SET XACT_ABORT, NOCOUNT ON;

			SET @o_deleted_params_rows = -1;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @v_params_keys TABLE (
					params_id BIGINT UNIQUE NOT NULL
				);
				DECLARE @v_orphans INT;

				INSERT INTO @v_params_keys ( params_id ) 
					SELECT TOP(500) [p].[PARAMS_ID] AS params_id 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] AS p WITH (NOLOCK) 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [p].[PARAMS_ID] = [msg].[PARAMS_ID] 
						);

				SET @v_orphans = @@ROWCOUNT;
	
				IF @v_orphans > 0
				BEGIN
					-- This time (with locks), try to delete our orphan candidates found during the non-blocking scan while transactionally ensuring they are truly orphans.
					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_PARAMS 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_PARAMS] AS p 
							INNER JOIN @v_params_keys AS pk ON [p].[PARAMS_ID] = [pk].[params_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [p].[PARAMS_ID] = [msg].[PARAMS_ID] 
						);

					SET @v_orphans = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_params_rows = @v_orphans;
				END
				ELSE
				BEGIN
					SET @o_deleted_params_rows = 0;
				END;

				RETURN 0;
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);
				DECLARE @v_error_severity INT;
				DECLARE @v_error_state INT;
				DECLARE @v_error INT;

				SELECT @v_error = @@ERROR, @v_error_message = ERROR_MESSAGE(), @v_error_severity = ERROR_SEVERITY(), @v_error_state = ERROR_STATE();

				IF @v_has_outer_transaction = 0
				BEGIN
					IF XACT_STATE() <> 0 
					BEGIN
						ROLLBACK TRANSACTION;
					END;
				END;

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
				RETURN -@v_error;
			END CATCH;
		END;
		'';

	EXECUTE dbo.sp_executesql @statement = @SQLString;

	IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_COMP]'', ''P'') IS NULL)
	BEGIN
		SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_COMP] AS SET NOCOUNT ON;''
		EXECUTE dbo.sp_executesql @statement = @SQLString; 
	END

	SET @SQLString = N''
		-- The purpose of this procedure is to delete orphaned log comp entries.
		-- This procedure should only be called with a world wide exclusitiviy lock managed by the caller.
		ALTER PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_COMP] (
			@o_deleted_comp_rows INT OUTPUT		-- Returns the number of comp rows deleted.
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;
	
			SET XACT_ABORT, NOCOUNT ON;

			SET @o_deleted_comp_rows = -1;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @v_comp_keys TABLE (
					comp_id BIGINT UNIQUE NOT NULL
				);
				DECLARE @v_orphans INT;

				INSERT INTO @v_comp_keys ( comp_id ) 
					SELECT TOP(500) [c].[COMP_ID] AS comp_id 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] AS c WITH (NOLOCK) 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [c].[COMP_ID] = [msg].[COMP_ID] 
						);

				SET @v_orphans = @@ROWCOUNT;
	
				IF @v_orphans > 0
				BEGIN
					-- This time (with locks), try to delete our orphan candidates found during the non-blocking scan while transactionally ensuring they are truly orphans.
					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_COMP 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_COMP] AS c 
							INNER JOIN @v_comp_keys AS ck ON [c].[COMP_ID] = [ck].[comp_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [c].[COMP_ID] = [msg].[COMP_ID] 
						);

					SET @v_orphans = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_comp_rows = @v_orphans;
				END
				ELSE
				BEGIN
					SET @o_deleted_comp_rows = 0;
				END;

				RETURN 0;
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);
				DECLARE @v_error_severity INT;
				DECLARE @v_error_state INT;
				DECLARE @v_error INT;

				SELECT @v_error = @@ERROR, @v_error_message = ERROR_MESSAGE(), @v_error_severity = ERROR_SEVERITY(), @v_error_state = ERROR_STATE();

				IF @v_has_outer_transaction = 0
				BEGIN
					IF XACT_STATE() <> 0 
					BEGIN
						ROLLBACK TRANSACTION;
					END;
				END;

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
				RETURN -@v_error;
			END CATCH;
		END;
		'';

	EXECUTE dbo.sp_executesql @statement = @SQLString;

	IF (OBJECT_ID (''[dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_SOURCE_IP]'', ''P'') IS NULL)
	BEGIN
		SET @SQLString = N''CREATE PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_SOURCE_IP] AS SET NOCOUNT ON;''
		EXECUTE dbo.sp_executesql @statement = @SQLString; 
	END

	SET @SQLString = N''
		-- The purpose of this procedure is to delete orphaned log source ip entries.
		-- This procedure should only be called with a world wide exclusitiviy lock managed by the caller.
		ALTER PROCEDURE [dbo].[DAL_LOG_SP_''+@ChannelName+''_LOG_EXPIRE_SOURCE_IP] (
			@o_deleted_source_ip_rows INT OUTPUT		-- Returns the number of source ip rows deleted.
		)
		AS
		BEGIN
			DECLARE @v_has_outer_transaction BIT;
	
			SET XACT_ABORT, NOCOUNT ON;

			SET @o_deleted_source_ip_rows = -1;

			SET @v_has_outer_transaction = CASE WHEN @@TRANCOUNT > 0 THEN 1 ELSE 0 END;

			BEGIN TRY
				DECLARE @v_source_ip_keys TABLE (
					source_ip_id BIGINT UNIQUE NOT NULL
				);
				DECLARE @v_orphans INT;

				INSERT INTO @v_source_ip_keys ( source_ip_id ) 
					SELECT TOP(500) [s].[SOURCE_IP_ID] AS source_ip_id 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] AS s WITH (NOLOCK) 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [s].[SOURCE_IP_ID] = [msg].[SOURCE_IP_ID] 
						);

				SET @v_orphans = @@ROWCOUNT;
	
				IF @v_orphans > 0
				BEGIN
					-- This time (with locks), try to delete our orphan candidates found during the non-blocking scan while transactionally ensuring they are truly orphans.
					IF @v_has_outer_transaction = 0
					BEGIN
						BEGIN TRANSACTION;
					END;

					DELETE DAL_T_''+@ChannelName+''_LOG_SOURCE_IP 
						FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_SOURCE_IP] AS s 
							INNER JOIN @v_source_ip_keys AS sk ON [s].[SOURCE_IP_ID] = [sk].[source_ip_id] 
						WHERE NOT EXISTS (
							SELECT 1 FROM [dbo].[DAL_T_''+@ChannelName+''_LOG_MESSAGE] AS msg WHERE [s].[SOURCE_IP_ID] = [msg].[SOURCE_IP_ID] 
						);

					SET @v_orphans = @@ROWCOUNT;

					IF @v_has_outer_transaction = 0
					BEGIN
						COMMIT TRANSACTION;
					END;

					SET @o_deleted_source_ip_rows = @v_orphans;
				END
				ELSE
				BEGIN
					SET @o_deleted_source_ip_rows = 0;
				END;

				RETURN 0;
			END TRY
			BEGIN CATCH
				DECLARE @v_error_message NVARCHAR(4000);
				DECLARE @v_error_severity INT;
				DECLARE @v_error_state INT;
				DECLARE @v_error INT;

				SELECT @v_error = @@ERROR, @v_error_message = ERROR_MESSAGE(), @v_error_severity = ERROR_SEVERITY(), @v_error_state = ERROR_STATE();

				IF @v_has_outer_transaction = 0
				BEGIN
					IF XACT_STATE() <> 0 
					BEGIN
						ROLLBACK TRANSACTION;
					END;
				END;

				RAISERROR (@v_error_message, @v_error_severity, @v_error_state);
				RETURN -@v_error;
			END CATCH;
		END;
		'';

	EXECUTE dbo.sp_executesql @statement = @SQLString;
END';

		COMMIT TRANSACTION;
	END TRY
	BEGIN CATCH
		PRINT '';
		PRINT '==============================================================';
		PRINT 'Code: ' + CONVERT(VARCHAR, ERROR_NUMBER());
		PRINT 'Message : ' + ERROR_MESSAGE();
		PRINT '==============================================================';

		IF (@@TRANCOUNT > 0)
		BEGIN
			ROLLBACK TRANSACTION;
		END
	END CATCH
END
